@@ -35,7 +35,6 @@ except ImportError:
 
 class _NamelistKeysView(KeysView):
     """Return the namelist's KeysView based on the Namelist iterator."""
-
     def __iter__(self):
         # This is faster, but requires Python 3.x
         # yield from self._mapping
@@ -47,7 +46,6 @@ class _NamelistKeysView(KeysView):
 
 class _NamelistItemsView(ItemsView):
     """Return the namelist's ItemsView based on the Namelist iterator."""
-
     def __iter__(self):
         for key in self._mapping:
             yield (key, self._mapping[key])
@@ -96,7 +94,8 @@ class Namelist(OrderedDict):
 
         # Construct the cogroups based on the internal key values
         self._cogroups = {}
-        for key in self:
+        for nmlkey in self:
+            key = nmlkey._key
             if key.startswith('_grp_'):
                 grp = key[5:].rsplit('_', 1)[0]
                 try:
@@ -143,7 +142,13 @@ class Namelist(OrderedDict):
 
     def __contains__(self, key):
         """Case-insensitive interface to OrderedDict."""
-        return super(Namelist, self).__contains__(key.lower())
+        lkey = key.lower()
+        # NOTE: Only Python 2.7 requires the hasattr() test.
+        #   I do not know why.  This needs more investigation.
+        return (
+            super(Namelist, self).__contains__(lkey)
+            or (hasattr(self, '_cogroups') and lkey in self._cogroups)
+        )
 
     def __delitem__(self, key):
         """Case-insensitive interface to OrderedDict."""
@@ -152,12 +157,10 @@ class Namelist(OrderedDict):
         if isinstance(key, NmlKey):
             super(Namelist, self).__delitem__(key._key)
         elif lkey in self._cogroups:
-            # Remove all cogroup values
-            cogrp = Cogroup(self, lkey)
-            for gkey in cogrp.keys:
+            for gkey in self._cogroups[lkey]:
                 super(Namelist, self).__delitem__(gkey)
 
-            self._cogroups.remove(lkey)
+            self._cogroups.pop(lkey)
         else:
             super(Namelist, self).__delitem__(key)
 
@@ -603,7 +606,7 @@ class Namelist(OrderedDict):
 
             # Remove and replace existing keys after the new cogroup key.
             for key in nml_groups[grp_idx+1:]:
-                grp = self.pop(key)
+                grp = self.pop(key._key)
                 self[key] = grp
 
             cogroup_keys = [cogrp_key]
@@ -616,7 +619,7 @@ class Namelist(OrderedDict):
         """Append a duplicate group to the Namelist as a new group."""
         lkey = key.lower()
 
-        if lkey in self and lkey not in self._cogroups:
+        if lkey not in self._cogroups:
             self.create_cogroup(lkey)
 
         # Generate the cogroup label and add to the Namelist
@@ -671,12 +674,7 @@ class Namelist(OrderedDict):
             sel = self
 
         for grp_name, grp_vars in sel.items():
-            # Check for repeated namelist records (saved as lists)
-            if isinstance(grp_vars, list):
-                for g_vars in grp_vars:
-                    self._write_nmlgrp(grp_name, g_vars, nml_file, sort)
-            else:
-                self._write_nmlgrp(grp_name, grp_vars, nml_file, sort)
+            self._write_nmlgrp(grp_name, grp_vars, nml_file, sort)
 
     def _write_nmlgrp(self, grp_name, grp_vars, nml_file, sort=False):
         """Write namelist group to target file."""
@@ -684,10 +682,6 @@ class Namelist(OrderedDict):
             print(file=nml_file)
         self._newline = True
 
-        # Strip metadata label for repeat groups
-        if grp_name.startswith('_grp_'):
-            grp_name = grp_name[5:].rsplit('_', 1)[0]
-
         if self.uppercase:
             grp_name = grp_name.upper()
 
@@ -883,7 +877,7 @@ class Namelist(OrderedDict):
                 val_strs.append(val_line.rstrip())
 
             # Final null values must always precede a comma
-            if val_strs and v_values[-1] is None:
+            if val_strs and (len(v_values) == 0 or v_values[-1] is None):
                 # NOTE: val_strs has been rstrip-ed so lead with a space
                 val_strs[-1] += ' ,'
 
@@ -909,12 +903,12 @@ class Namelist(OrderedDict):
         # Copy Namelist to OrderedDict, converting NmlKeys back to strings
         nmldict = OrderedDict()
         for key, value in self.items():
-            nmldict[str(key)] = value
+            nmldict[str(key._key)] = value
 
         # Search for namelists within the namelist
         # TODO: Move repeated stuff to new functions
         for nkey, value in self.items():
-            key = str(nkey)
+            key = str(nkey._key)
 
             if isinstance(value, Namelist):
                 nml = copy.deepcopy(value)
@@ -1044,15 +1038,18 @@ class Cogroup(list):
         an element is deleted from the Cogroup, it is passed to the namelist
         and deleted.
         """
-        key = self.keys[index]
-        del self.nml[key]
+        # First remove the actual key from the namelist
+        nmlkey = self.keys[index]
+        del self.nml[nmlkey]
+
+        # Next update the namelist's internal cogroup record
+        self.nml._cogroups[self.key].remove(nmlkey._key)
+        if not self.nml._cogroups[self.key]:
+            del self.nml._cogroups[self.key]
+
+        # Finally, remove from this list
         super(Cogroup, self).__delitem__(index)
 
-        # Remove the cogroup status if keys are depleted
-        if len(self) == 0:
-            self.nml._cogroups.remove(self.key)
-
-    # NOTE: Maybe this function is no longer useful...
     @property
     def keys(self):
         """Return the namelist keys in the cogroup."""
